#pragma once

//The code examples are written in C++ / CLI for the.Net Framework 4.0

public ref class Camera
{
	public:
		Camera();
		virtual ~Camera();

		// Connect / Disconnect
		System::Boolean Connect(System::String^ ip_address);
		System::Void Disconnect(System::Void);

		// Resolution
		System::Boolean SetResolution(System::Int32 x, System::Int32 y);
		System::Boolean GetResolution(System::Int32% x, System::Int32% y);
		System::Boolean GetSensorScale(System::Int32% x_min, System::Int32% x_max, System::Int32% x_inc, System::Int32% y_min, System::Int32% y_max, System::Int32% y_inc);

		// Exposure
		System::Boolean SetExposureTime(System::Int32 exp_in_us);
		System::Boolean GetExposureTime(System::Int32% exp_in_us);

		// Framerate
		System::Boolean SetFramerate(System::Int32 fps);
		System::Boolean GetFramerate(System::Int32% fps);

		// Live
		System::Boolean LiveStart(System::Boolean lowResolution);
		System::Boolean LiveStop(System::Void);
		System::Boolean GetFrame(unsigned char* buffer, System::Int32 buffer_size);

		// Recording
		System::Boolean RecordSetup(System::Int32 numberOfFrames, System::Int32 trigger, System::Int32 triggerPos);
		System::Boolean RecordStart(System::Void);
		System::Boolean RecordStop(System::Void);
		System::Boolean RecordAbort(System::Void);
		System::Boolean SoftwareTrigger(System::Void);
		System::Boolean EnableSoftwareTrigger(System::Void);
		System::Boolean DisableSoftwareTrigger(System::Void);
		System::Boolean GetRecordingStatus(System::Int32% status);

		// Play
		System::Boolean PlayStart(System::Void);
		System::Boolean PlayStop(System::Void);
		System::Boolean GetFrameMemory(unsigned char* buffer, System::Int32 buffer_size, System::Int32 frame_number);
		System::Boolean GetFrameMemory(unsigned char* buffer, System::Int32 buffer_size, System::UInt64% time_marker, System::Int32 frame_number);
		System::Boolean GetSequenceParameter(System::Int32% frames, System::Int32% resoX, System::Int32% resoY, System::Int32% fps, System::Int32% exptime, System::Int32% trigpos, System::Boolean% bFG8bits, System::Int32% mode, System::Int32% nbSeq, System::Double% realfps);
		System::Boolean IsSequenceAvailable(System::Boolean% available);
		
	private:
		System::Void LockCamera(System::Void);
		System::Void UnlockCamera(System::Void);

		// Open and close the network connection to the camera
		System::Boolean NetworkConnect(System::String^ ip_address, System::Int32 port);
		System::Void NetworkDisconnect(System::Void);

		// Communicate with the camera
		System::Boolean NetworkSend(System::String^ message);
		System::Boolean NetworkReceive(System::Collections::Generic::List<System::String^>% received);
		System::Boolean NetworkReceiveImage(unsigned char* buffer, System::Int32 buffer_size);

		// Members
		System::Threading::Mutex^ m_lockCamera;

		System::Net::Sockets::TcpClient^ m_networkSocket;
		System::Net::Sockets::NetworkStream^ m_networkStream;
};