#define SUCCESS							0
#define ERROR_SSD_NOT_OPEN				1
#define ERROR_NO_SSD_DETECTED			2
#define ERROR_NO_SEQUENCE_FOUND			3
#define ERROR_SSD_RD					4
#define ERROR_SSD_RANGE_PB				5
#define ERROR_SSD_WRITE_PB				6
#define ERROR_DLL_PB					7


///////////////////////////////////////////////////////////////////////////////////////////
// int exSSD_IsDrive()
// Check if a SSD drive is connected
// Returns SUCCESS if a SSD drive is connected to the PC
// Returns ERROR_NO_SSD_DETECTED if no SSD drive is found
///////////////////////////////////////////////////////////////////////////////////////////
extern "C" __declspec(dllexport) int __cdecl exSSD_IsDrive();

///////////////////////////////////////////////////////////////////////////////////////////
// int exSSD_OpenDrive()
// Open the SSD drive
// Returns SUCCESS if the SSD drive has been successfully open 
// Returns ERROR_NO_SSD_DETECTED if no SSD drive is found
// Returns ERROR_SSD_NOT_OPEN if SSD drive opening process went wrong
// Returns ERROR_NO_SEQUENCE_FOUND if no sequence is stored on the SSD drive
///////////////////////////////////////////////////////////////////////////////////////////
extern "C" __declspec(dllexport) int __cdecl exSSD_OpenDrive();



///////////////////////////////////////////////////////////////////////////////////////////
// int exSSD_GetSeqNumber(__int64 *n)
// Returns the number of sequence stored in the SSD drive
// Returns SUCCESS and n = number of sequence stored in the SSD drive
// Returns ERROR_SSD_NOT_OPEN if SSD drive opening process went wrong
///////////////////////////////////////////////////////////////////////////////////////////
extern "C" __declspec(dllexport) int __cdecl exSSD_GetSeqNumber(__int64 *n);

///////////////////////////////////////////////////////////////////////////////////////////
// int exSSD_SetCurrentSeq(__int64 seq)
// Point to a given sequence in the SSD
// Returns SUCCESS
// seq is the sequence number to access
///////////////////////////////////////////////////////////////////////////////////////////
extern "C" __declspec(dllexport) int __cdecl exSSD_SetCurrentSeq(__int64 seq);

///////////////////////////////////////////////////////////////////////////////////////////
// int exSSD_GetSeqHeader(__int64 seq,long *frames,int *resoX,int *resoY,long *fps,long *exptime,long *trigpos,int *mode,__int64 *realfpsHex,int *bFG8bits,unsigned char *Cam_SerialN)
// Gets the header of the current sequence pointed by exSSD_SetCurrentSeq
// Returns SUCCESS
// seq is the sequence number to access
// frames is the number of frames stored in the sequence
// resoX is the frame width
// resoY is the frame height
// fps is the framerate in frames per second
// exptime is the exposure time in us
// trigpos is the trigger position within the sequence
// mode is the acquisition mode
// realfpsHex is the real framerate (TO BE DEFINED)
// bFG8bits is the pixel depth : 0 for 10bits; 1 for 8bits
// Cam_SerialN is the camera serial number
///////////////////////////////////////////////////////////////////////////////////////////
extern "C" __declspec(dllexport) int __cdecl exSSD_GetSeqHeader(__int64 seq,long *frames,int *resoX,int *resoY,long *fps,long *exptime,long *trigpos,int *mode,
																__int64 *realfpsHex,int *bFG8bits,unsigned char *Cam_SerialN, int *bSensorColor, int *sensorBayer,
																int *ssdTimeMarker_en, int *recordLeft, int *recordTop);

extern "C" __declspec(dllexport) int __cdecl exSSD_GetSeqHeaderTrigTM(__int64 seq,long *frames,int *resoX,int *resoY,long *fps,long *exptime,long *trigpos,int *mode,
																__int64 *realfpsHex,int *bFG8bits,unsigned char *Cam_SerialN, int *bSensorColor, int *sensorBayer,
																int *ssdTimeMarker_en, int *recordLeft, int *recordTop, __int64 *trigTM,
																bool *wbCoeffAvailable, int *wbCoeff1, int *wbCoeff2, int *wbCoeff3, int *wbCoeff4);

extern "C" __declspec(dllexport) int __cdecl exSSD_GetUserInfo(int *userInfoSize, unsigned char *userInfo);

extern "C" __declspec(dllexport) int __cdecl exSSD_EditUserInfo(char *userInfo);

//__declspec(dllexport) 
/*extern "C"
{
	ExtSSDSeqHeader;
};
extern "C" __declspec(dllexport) int __cdecl exSSD_GetSeqHeader_st(ExtSSDSeqHeader_st *header);*/

extern "C" __declspec(dllexport) int __cdecl exSSD_GetSeqAddr(__int64 *addr);

///////////////////////////////////////////////////////////////////////////////////////////
// int exSSD_GetFrame(__int64 frame, unsigned char *pimage)
// Get a frame on the current sequence pointed by exSSD_SetCurrentSeq
// Returns SUCCESS
// frame is the frame ID to read on the SSD
// pimage is the destination buffer
///////////////////////////////////////////////////////////////////////////////////////////
extern "C" __declspec(dllexport) int __cdecl exSSD_GetFrame(__int64* timeMarker, __int64 frame, unsigned char *pimage);

///////////////////////////////////////////////////////////////////////////////////////////
// int exSSD_Format(int option)
// formart ssd to optronis format
///////////////////////////////////////////////////////////////////////////////////////////
//extern "C" __declspec(dllexport) int __cdecl exSSD_Format(int option=0);

///////////////////////////////////////////////////////////////////////////////////////////
// int exSSD_GetDriveList
///////////////////////////////////////////////////////////////////////////////////////////
extern "C" __declspec(dllexport) int __cdecl exSSD_GetDriveList();

///////////////////////////////////////////////////////////////////////////////////////////
// int exSSD_DelNSeq
///////////////////////////////////////////////////////////////////////////////////////////
//extern "C" __declspec(dllexport) int __cdecl exSSD_EraseSeq();
extern "C" __declspec(dllexport) int __cdecl exSSD_DelNSeq(int _seqToDelNb);



extern "C" __declspec(dllexport) int __cdecl test(int param);
