#include "stdafx.h"
#include "ui_main.h"

System::Void CamRecordSprinterDemo::ui_main::ActualizeFormStatus(System::Void)
{
	// Activate / Deactivate controls, dependend of the camera connection and mode
	m_btn_connect->Enabled		= !m_connected;
	m_btn_disconnect->Enabled	= m_connected;

	m_gb_control->Enabled		= m_connected;
	m_btn_live_start->Enabled	= !m_live_mode;
	m_btn_live_stop->Enabled	= m_live_mode;

	m_gb_properties->Enabled	= m_connected && !m_live_mode;
}

System::Void CamRecordSprinterDemo::ui_main::ActualizeControls(System::Void)
{
	// Read camera parameters (Resolution, exposure and fps) and actualize controls
	if (m_camera != nullptr)
	{
		if (m_connected && !m_live_mode)
		{
			System::Int32 res_x(0), res_y(0), exposure(0), fps(0);

			// Read values from camera
			m_camera->GetResolution(res_x, res_y);
			m_camera->GetExposureTime(exposure);
			m_camera->GetFramerate(fps);

			// Actualize controls
			m_tb_x->Text		= System::Convert::ToString(res_x);
			m_tb_y->Text		= System::Convert::ToString(res_y);
			m_tb_exposure->Text	= System::Convert::ToString(exposure / 1000);
			m_tb_fps->Text		= System::Convert::ToString(fps);
		}
		else
		{
			m_tb_x->Text		= "-";
			m_tb_y->Text		= "-";
			m_tb_exposure->Text	= "-";
			m_tb_fps->Text		= "-";
		}
	}
}

System::Void CamRecordSprinterDemo::ui_main::ReadImage(System::Void)
{
	if (InvokeRequired)
	{
		ReadImageDelegate^ d = gcnew ReadImageDelegate(this, &CamRecordSprinterDemo::ui_main::ReadImage);
		this->Invoke(d);
	}
	else
	{
		if (m_camera != nullptr)
		{
			unsigned char *buf;
			
			// Get BitmapData object from the displayed image
			Imaging::BitmapData^ bitmapData = m_image->LockBits(Drawing::Rectangle(0, 0, m_image->Width, m_image->Height), Imaging::ImageLockMode::ReadWrite, m_image->PixelFormat);

			// Set unsigned char pointer 'buf' to the data pointer from the System::Drawing::Bitmap object
			buf = reinterpret_cast<unsigned char *>(bitmapData->Scan0.ToPointer());

			// Read image from the camera
			m_camera->GetFrame(buf, m_image_width * m_image_height);

			m_image->UnlockBits(bitmapData);

			// Refresh display
			m_pb_image->Refresh();
		}		
	}
}