#pragma once

#include "Camera.h"

namespace CamRecordSprinterDemo {

	using namespace System;
	using namespace System::ComponentModel;
	using namespace System::Collections;
	using namespace System::Windows::Forms;
	using namespace System::Data;
	using namespace System::Drawing;

	/// <summary>
	/// Summary for ui_main
	/// </summary>
	public ref class ui_main : public System::Windows::Forms::Form 
	{
	public:
		ui_main(void) :
			m_camera(nullptr),
			m_connected(false),
			m_live_mode(false),
			m_stop_thread(false),
			m_thread_stopped(false),
			m_image(nullptr),
			m_image_width(0),
			m_image_height(0),
			m_x_min(0),
			m_x_max(0),
			m_x_inc(0),
			m_y_min(0),
			m_y_max(0),
			m_y_inc(0)
		{
			InitializeComponent();
			
			m_cb_viewmode->SelectedIndex = 0;

			m_camera = gcnew Camera;

			ActualizeFormStatus();
		}

	protected:
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		~ui_main()
		{
			if (m_camera != nullptr)
			{
				if (m_connected)
				{
					if (m_live_mode)
					{
						m_camera->LiveStop();
					}

					m_camera->Disconnect();
				}

				delete m_camera;
			}

			if (components)
			{
				delete components;
			}
		}

	private: System::Windows::Forms::Panel^  m_panel_controls;
	private: System::Windows::Forms::Panel^  m_panel_image;

	private: System::Windows::Forms::Button^  m_btn_connect;
	private: System::Windows::Forms::Button^  m_btn_disconnect;	
	private: System::Windows::Forms::Button^  m_btn_live_stop;
	private: System::Windows::Forms::Button^  m_btn_live_start;
	private: System::Windows::Forms::Button^  m_btn_fps_send;
	private: System::Windows::Forms::Button^  m_btn_x_send;
	private: System::Windows::Forms::Button^  m_btn_y_send;
	private: System::Windows::Forms::Button^  m_btn_exposure_send;

	private: System::Windows::Forms::GroupBox^  m_gb_display;
	private: System::Windows::Forms::GroupBox^  m_gb_properties;
	private: System::Windows::Forms::GroupBox^  m_gb_control;
	private: System::Windows::Forms::GroupBox^  m_gb_camera;

	private: System::Windows::Forms::ComboBox^  m_cb_viewmode;

	private: System::Windows::Forms::Label^  m_lb_viewmode;
	private: System::Windows::Forms::Label^  m_lb_fps_unit;
	private: System::Windows::Forms::Label^  m_lb_exposure_unit;
	private: System::Windows::Forms::Label^  m_lb_y_unit;
	private: System::Windows::Forms::Label^  m_lb_y;
	private: System::Windows::Forms::Label^  m_lb_x_unit;
	private: System::Windows::Forms::Label^  m_lb_fps;
	private: System::Windows::Forms::Label^  m_lb_x;
	private: System::Windows::Forms::Label^  m_lb_exposure;
	private: System::Windows::Forms::Label^  m_lb_resolution;
	private: System::Windows::Forms::Label^  m_lb_ip;
	private: System::Windows::Forms::Label^  m_lb_pt3;
	private: System::Windows::Forms::Label^  m_lb_pt2;
	private: System::Windows::Forms::Label^  m_lb_pt1;
	
	private: System::Windows::Forms::TextBox^  m_tb_fps;
	private: System::Windows::Forms::TextBox^  m_tb_exposure;
	private: System::Windows::Forms::TextBox^  m_tb_y;
	private: System::Windows::Forms::TextBox^  m_tb_x;
	private: System::Windows::Forms::TextBox^  m_tb_ip_p2;
	private: System::Windows::Forms::TextBox^  m_tb_ip_p4;
	private: System::Windows::Forms::TextBox^  m_tb_ip_p3;
	private: System::Windows::Forms::TextBox^  m_tb_ip_p1;

	private: System::Windows::Forms::PictureBox^  m_pb_image;
			 
	private: System::ComponentModel::BackgroundWorker^  m_thread_read_image;

	private: Camera^ m_camera;

	private: System::Boolean m_connected;
	private: System::Boolean m_live_mode;
	private: System::Boolean m_stop_thread;
	private: System::Boolean m_thread_stopped;

	private: System::Drawing::Bitmap^ m_image;
	
	private: System::Int32 m_image_width;
	private: System::Int32 m_image_height;
	private: System::Int32 m_x_min;
	private: System::Int32 m_x_max;
	private: System::Int32 m_x_inc;
	private: System::Int32 m_y_min;
	private: System::Int32 m_y_max;
	private: System::Int32 m_y_inc;

	private: System::Void ActualizeFormStatus(System::Void);
	private: System::Void ActualizeControls(System::Void);
	
	private: System::Void ReadImage(System::Void);
	private: delegate System::Void ReadImageDelegate(System::Void);

	private:
		/// <summary>
		/// Required designer variable.
		/// </summary>
		System::ComponentModel::Container ^components;

#pragma region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
			this->m_panel_controls = (gcnew System::Windows::Forms::Panel());
			this->m_gb_display = (gcnew System::Windows::Forms::GroupBox());
			this->m_cb_viewmode = (gcnew System::Windows::Forms::ComboBox());
			this->m_lb_viewmode = (gcnew System::Windows::Forms::Label());
			this->m_gb_properties = (gcnew System::Windows::Forms::GroupBox());
			this->m_btn_fps_send = (gcnew System::Windows::Forms::Button());
			this->m_lb_fps_unit = (gcnew System::Windows::Forms::Label());
			this->m_tb_fps = (gcnew System::Windows::Forms::TextBox());
			this->m_btn_exposure_send = (gcnew System::Windows::Forms::Button());
			this->m_lb_exposure_unit = (gcnew System::Windows::Forms::Label());
			this->m_tb_exposure = (gcnew System::Windows::Forms::TextBox());
			this->m_btn_y_send = (gcnew System::Windows::Forms::Button());
			this->m_lb_y_unit = (gcnew System::Windows::Forms::Label());
			this->m_tb_y = (gcnew System::Windows::Forms::TextBox());
			this->m_lb_y = (gcnew System::Windows::Forms::Label());
			this->m_btn_x_send = (gcnew System::Windows::Forms::Button());
			this->m_lb_x_unit = (gcnew System::Windows::Forms::Label());
			this->m_tb_x = (gcnew System::Windows::Forms::TextBox());
			this->m_lb_fps = (gcnew System::Windows::Forms::Label());
			this->m_lb_exposure = (gcnew System::Windows::Forms::Label());
			this->m_lb_x = (gcnew System::Windows::Forms::Label());
			this->m_lb_resolution = (gcnew System::Windows::Forms::Label());
			this->m_gb_control = (gcnew System::Windows::Forms::GroupBox());
			this->m_btn_live_stop = (gcnew System::Windows::Forms::Button());
			this->m_btn_live_start = (gcnew System::Windows::Forms::Button());
			this->m_gb_camera = (gcnew System::Windows::Forms::GroupBox());
			this->m_lb_pt3 = (gcnew System::Windows::Forms::Label());
			this->m_lb_pt2 = (gcnew System::Windows::Forms::Label());
			this->m_lb_pt1 = (gcnew System::Windows::Forms::Label());
			this->m_tb_ip_p2 = (gcnew System::Windows::Forms::TextBox());
			this->m_tb_ip_p4 = (gcnew System::Windows::Forms::TextBox());
			this->m_tb_ip_p3 = (gcnew System::Windows::Forms::TextBox());
			this->m_tb_ip_p1 = (gcnew System::Windows::Forms::TextBox());
			this->m_lb_ip = (gcnew System::Windows::Forms::Label());
			this->m_btn_disconnect = (gcnew System::Windows::Forms::Button());
			this->m_btn_connect = (gcnew System::Windows::Forms::Button());
			this->m_panel_image = (gcnew System::Windows::Forms::Panel());
			this->m_pb_image = (gcnew System::Windows::Forms::PictureBox());
			this->m_thread_read_image = (gcnew System::ComponentModel::BackgroundWorker());
			this->m_panel_controls->SuspendLayout();
			this->m_gb_display->SuspendLayout();
			this->m_gb_properties->SuspendLayout();
			this->m_gb_control->SuspendLayout();
			this->m_gb_camera->SuspendLayout();
			this->m_panel_image->SuspendLayout();
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^>(this->m_pb_image))->BeginInit();
			this->SuspendLayout();
			// 
			// m_panel_controls
			// 
			this->m_panel_controls->Controls->Add(this->m_gb_display);
			this->m_panel_controls->Controls->Add(this->m_gb_properties);
			this->m_panel_controls->Controls->Add(this->m_gb_control);
			this->m_panel_controls->Controls->Add(this->m_gb_camera);
			this->m_panel_controls->Dock = System::Windows::Forms::DockStyle::Left;
			this->m_panel_controls->Location = System::Drawing::Point(0, 0);
			this->m_panel_controls->Name = L"m_panel_controls";
			this->m_panel_controls->Size = System::Drawing::Size(180, 754);
			this->m_panel_controls->TabIndex = 0;
			// 
			// m_gb_display
			// 
			this->m_gb_display->Anchor = static_cast<System::Windows::Forms::AnchorStyles>(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Left)
				| System::Windows::Forms::AnchorStyles::Right));
			this->m_gb_display->Controls->Add(this->m_cb_viewmode);
			this->m_gb_display->Controls->Add(this->m_lb_viewmode);
			this->m_gb_display->Location = System::Drawing::Point(3, 416);
			this->m_gb_display->Name = L"m_gb_display";
			this->m_gb_display->Size = System::Drawing::Size(171, 44);
			this->m_gb_display->TabIndex = 20;
			this->m_gb_display->TabStop = false;
			this->m_gb_display->Text = L"Display";
			// 
			// m_cb_viewmode
			// 
			this->m_cb_viewmode->DropDownStyle = System::Windows::Forms::ComboBoxStyle::DropDownList;
			this->m_cb_viewmode->FormattingEnabled = true;
			this->m_cb_viewmode->Items->AddRange(gcnew cli::array< System::Object^  >(3) { L"Normal", L"Center Image", L"Stretch" });
			this->m_cb_viewmode->Location = System::Drawing::Point(76, 13);
			this->m_cb_viewmode->Name = L"m_cb_viewmode";
			this->m_cb_viewmode->Size = System::Drawing::Size(89, 21);
			this->m_cb_viewmode->TabIndex = 17;
			this->m_cb_viewmode->SelectedIndexChanged += gcnew System::EventHandler(this, &ui_main::m_cb_viewmode_SelectedIndexChanged);
			// 
			// m_lb_viewmode
			// 
			this->m_lb_viewmode->AutoSize = true;
			this->m_lb_viewmode->Location = System::Drawing::Point(9, 16);
			this->m_lb_viewmode->Name = L"m_lb_viewmode";
			this->m_lb_viewmode->Size = System::Drawing::Size(65, 13);
			this->m_lb_viewmode->TabIndex = 8;
			this->m_lb_viewmode->Text = L"View mode:";
			// 
			// m_gb_properties
			// 
			this->m_gb_properties->Anchor = static_cast<System::Windows::Forms::AnchorStyles>(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Left)
				| System::Windows::Forms::AnchorStyles::Right));
			this->m_gb_properties->Controls->Add(this->m_btn_fps_send);
			this->m_gb_properties->Controls->Add(this->m_lb_fps_unit);
			this->m_gb_properties->Controls->Add(this->m_tb_fps);
			this->m_gb_properties->Controls->Add(this->m_btn_exposure_send);
			this->m_gb_properties->Controls->Add(this->m_lb_exposure_unit);
			this->m_gb_properties->Controls->Add(this->m_tb_exposure);
			this->m_gb_properties->Controls->Add(this->m_btn_y_send);
			this->m_gb_properties->Controls->Add(this->m_lb_y_unit);
			this->m_gb_properties->Controls->Add(this->m_tb_y);
			this->m_gb_properties->Controls->Add(this->m_lb_y);
			this->m_gb_properties->Controls->Add(this->m_btn_x_send);
			this->m_gb_properties->Controls->Add(this->m_lb_x_unit);
			this->m_gb_properties->Controls->Add(this->m_tb_x);
			this->m_gb_properties->Controls->Add(this->m_lb_fps);
			this->m_gb_properties->Controls->Add(this->m_lb_exposure);
			this->m_gb_properties->Controls->Add(this->m_lb_x);
			this->m_gb_properties->Controls->Add(this->m_lb_resolution);
			this->m_gb_properties->Location = System::Drawing::Point(3, 225);
			this->m_gb_properties->Name = L"m_gb_properties";
			this->m_gb_properties->Size = System::Drawing::Size(171, 185);
			this->m_gb_properties->TabIndex = 4;
			this->m_gb_properties->TabStop = false;
			this->m_gb_properties->Text = L"Properties";
			// 
			// m_btn_fps_send
			// 
			this->m_btn_fps_send->Anchor = static_cast<System::Windows::Forms::AnchorStyles>((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Right));
			this->m_btn_fps_send->Location = System::Drawing::Point(117, 155);
			this->m_btn_fps_send->Name = L"m_btn_fps_send";
			this->m_btn_fps_send->Size = System::Drawing::Size(45, 23);
			this->m_btn_fps_send->TabIndex = 16;
			this->m_btn_fps_send->Text = L"Send";
			this->m_btn_fps_send->UseVisualStyleBackColor = true;
			this->m_btn_fps_send->Click += gcnew System::EventHandler(this, &ui_main::m_btn_fps_send_Click);
			// 
			// m_lb_fps_unit
			// 
			this->m_lb_fps_unit->AutoSize = true;
			this->m_lb_fps_unit->Location = System::Drawing::Point(84, 160);
			this->m_lb_fps_unit->Name = L"m_lb_fps_unit";
			this->m_lb_fps_unit->Size = System::Drawing::Size(21, 13);
			this->m_lb_fps_unit->TabIndex = 19;
			this->m_lb_fps_unit->Text = L"fps";
			// 
			// m_tb_fps
			// 
			this->m_tb_fps->Location = System::Drawing::Point(37, 157);
			this->m_tb_fps->Name = L"m_tb_fps";
			this->m_tb_fps->Size = System::Drawing::Size(41, 20);
			this->m_tb_fps->TabIndex = 15;
			this->m_tb_fps->Text = L"0";
			this->m_tb_fps->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->m_tb_fps->KeyPress += gcnew System::Windows::Forms::KeyPressEventHandler(this, &ui_main::m_tb_fps_KeyPress);
			// 
			// m_btn_exposure_send
			// 
			this->m_btn_exposure_send->Anchor = static_cast<System::Windows::Forms::AnchorStyles>((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Right));
			this->m_btn_exposure_send->Location = System::Drawing::Point(117, 103);
			this->m_btn_exposure_send->Name = L"m_btn_exposure_send";
			this->m_btn_exposure_send->Size = System::Drawing::Size(45, 23);
			this->m_btn_exposure_send->TabIndex = 14;
			this->m_btn_exposure_send->Text = L"Send";
			this->m_btn_exposure_send->UseVisualStyleBackColor = true;
			this->m_btn_exposure_send->Click += gcnew System::EventHandler(this, &ui_main::m_btn_exposure_send_Click);
			// 
			// m_lb_exposure_unit
			// 
			this->m_lb_exposure_unit->AutoSize = true;
			this->m_lb_exposure_unit->Location = System::Drawing::Point(84, 108);
			this->m_lb_exposure_unit->Name = L"m_lb_exposure_unit";
			this->m_lb_exposure_unit->Size = System::Drawing::Size(20, 13);
			this->m_lb_exposure_unit->TabIndex = 16;
			this->m_lb_exposure_unit->Text = L"ms";
			// 
			// m_tb_exposure
			// 
			this->m_tb_exposure->Location = System::Drawing::Point(37, 105);
			this->m_tb_exposure->Name = L"m_tb_exposure";
			this->m_tb_exposure->Size = System::Drawing::Size(41, 20);
			this->m_tb_exposure->TabIndex = 13;
			this->m_tb_exposure->Text = L"0";
			this->m_tb_exposure->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->m_tb_exposure->KeyPress += gcnew System::Windows::Forms::KeyPressEventHandler(this, &ui_main::m_tb_exposure_KeyPress);
			// 
			// m_btn_y_send
			// 
			this->m_btn_y_send->Anchor = static_cast<System::Windows::Forms::AnchorStyles>((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Right));
			this->m_btn_y_send->Location = System::Drawing::Point(117, 54);
			this->m_btn_y_send->Name = L"m_btn_y_send";
			this->m_btn_y_send->Size = System::Drawing::Size(45, 23);
			this->m_btn_y_send->TabIndex = 12;
			this->m_btn_y_send->Text = L"Send";
			this->m_btn_y_send->UseVisualStyleBackColor = true;
			this->m_btn_y_send->Click += gcnew System::EventHandler(this, &ui_main::SetResolution_Click);
			// 
			// m_lb_y_unit
			// 
			this->m_lb_y_unit->AutoSize = true;
			this->m_lb_y_unit->Location = System::Drawing::Point(85, 59);
			this->m_lb_y_unit->Name = L"m_lb_y_unit";
			this->m_lb_y_unit->Size = System::Drawing::Size(29, 13);
			this->m_lb_y_unit->TabIndex = 13;
			this->m_lb_y_unit->Text = L"Pixel";
			// 
			// m_tb_y
			// 
			this->m_tb_y->Location = System::Drawing::Point(38, 56);
			this->m_tb_y->Name = L"m_tb_y";
			this->m_tb_y->Size = System::Drawing::Size(41, 20);
			this->m_tb_y->TabIndex = 11;
			this->m_tb_y->Text = L"0";
			this->m_tb_y->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->m_tb_y->KeyPress += gcnew System::Windows::Forms::KeyPressEventHandler(this, &ui_main::m_tb_y_KeyPress);
			// 
			// m_lb_y
			// 
			this->m_lb_y->AutoSize = true;
			this->m_lb_y->Location = System::Drawing::Point(15, 59);
			this->m_lb_y->Name = L"m_lb_y";
			this->m_lb_y->Size = System::Drawing::Size(17, 13);
			this->m_lb_y->TabIndex = 11;
			this->m_lb_y->Text = L"Y:";
			// 
			// m_btn_x_send
			// 
			this->m_btn_x_send->Anchor = static_cast<System::Windows::Forms::AnchorStyles>((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Right));
			this->m_btn_x_send->Location = System::Drawing::Point(117, 30);
			this->m_btn_x_send->Name = L"m_btn_x_send";
			this->m_btn_x_send->Size = System::Drawing::Size(45, 23);
			this->m_btn_x_send->TabIndex = 10;
			this->m_btn_x_send->Text = L"Send";
			this->m_btn_x_send->UseVisualStyleBackColor = true;
			this->m_btn_x_send->Click += gcnew System::EventHandler(this, &ui_main::SetResolution_Click);
			// 
			// m_lb_x_unit
			// 
			this->m_lb_x_unit->AutoSize = true;
			this->m_lb_x_unit->Location = System::Drawing::Point(85, 35);
			this->m_lb_x_unit->Name = L"m_lb_x_unit";
			this->m_lb_x_unit->Size = System::Drawing::Size(29, 13);
			this->m_lb_x_unit->TabIndex = 9;
			this->m_lb_x_unit->Text = L"Pixel";
			// 
			// m_tb_x
			// 
			this->m_tb_x->Location = System::Drawing::Point(38, 32);
			this->m_tb_x->Name = L"m_tb_x";
			this->m_tb_x->Size = System::Drawing::Size(41, 20);
			this->m_tb_x->TabIndex = 9;
			this->m_tb_x->Text = L"0";
			this->m_tb_x->TextAlign = System::Windows::Forms::HorizontalAlignment::Right;
			this->m_tb_x->KeyPress += gcnew System::Windows::Forms::KeyPressEventHandler(this, &ui_main::m_tb_x_KeyPress);
			// 
			// m_lb_fps
			// 
			this->m_lb_fps->AutoSize = true;
			this->m_lb_fps->Location = System::Drawing::Point(9, 141);
			this->m_lb_fps->Name = L"m_lb_fps";
			this->m_lb_fps->Size = System::Drawing::Size(51, 13);
			this->m_lb_fps->TabIndex = 7;
			this->m_lb_fps->Text = L"Framrate:";
			// 
			// m_lb_exposure
			// 
			this->m_lb_exposure->AutoSize = true;
			this->m_lb_exposure->Location = System::Drawing::Point(9, 89);
			this->m_lb_exposure->Name = L"m_lb_exposure";
			this->m_lb_exposure->Size = System::Drawing::Size(51, 13);
			this->m_lb_exposure->TabIndex = 6;
			this->m_lb_exposure->Text = L"Exposure";
			// 
			// m_lb_x
			// 
			this->m_lb_x->AutoSize = true;
			this->m_lb_x->Location = System::Drawing::Point(15, 35);
			this->m_lb_x->Name = L"m_lb_x";
			this->m_lb_x->Size = System::Drawing::Size(17, 13);
			this->m_lb_x->TabIndex = 5;
			this->m_lb_x->Text = L"X:";
			// 
			// m_lb_resolution
			// 
			this->m_lb_resolution->AutoSize = true;
			this->m_lb_resolution->Location = System::Drawing::Point(9, 16);
			this->m_lb_resolution->Name = L"m_lb_resolution";
			this->m_lb_resolution->Size = System::Drawing::Size(57, 13);
			this->m_lb_resolution->TabIndex = 4;
			this->m_lb_resolution->Text = L"Resolution";
			// 
			// m_gb_control
			// 
			this->m_gb_control->Anchor = static_cast<System::Windows::Forms::AnchorStyles>(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Left)
				| System::Windows::Forms::AnchorStyles::Right));
			this->m_gb_control->Controls->Add(this->m_btn_live_stop);
			this->m_gb_control->Controls->Add(this->m_btn_live_start);
			this->m_gb_control->Location = System::Drawing::Point(3, 140);
			this->m_gb_control->Name = L"m_gb_control";
			this->m_gb_control->Size = System::Drawing::Size(171, 79);
			this->m_gb_control->TabIndex = 3;
			this->m_gb_control->TabStop = false;
			this->m_gb_control->Text = L"Control";
			// 
			// m_btn_live_stop
			// 
			this->m_btn_live_stop->Anchor = static_cast<System::Windows::Forms::AnchorStyles>((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Right));
			this->m_btn_live_stop->Location = System::Drawing::Point(9, 48);
			this->m_btn_live_stop->Name = L"m_btn_live_stop";
			this->m_btn_live_stop->Size = System::Drawing::Size(153, 23);
			this->m_btn_live_stop->TabIndex = 8;
			this->m_btn_live_stop->Text = L"Stop Live";
			this->m_btn_live_stop->UseVisualStyleBackColor = true;
			this->m_btn_live_stop->Click += gcnew System::EventHandler(this, &ui_main::m_btn_live_stop_Click);
			// 
			// m_btn_live_start
			// 
			this->m_btn_live_start->Anchor = static_cast<System::Windows::Forms::AnchorStyles>((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Right));
			this->m_btn_live_start->Location = System::Drawing::Point(9, 19);
			this->m_btn_live_start->Name = L"m_btn_live_start";
			this->m_btn_live_start->Size = System::Drawing::Size(153, 23);
			this->m_btn_live_start->TabIndex = 7;
			this->m_btn_live_start->Text = L"Start Live";
			this->m_btn_live_start->UseVisualStyleBackColor = true;
			this->m_btn_live_start->Click += gcnew System::EventHandler(this, &ui_main::m_btn_live_start_Click);
			// 
			// m_gb_camera
			// 
			this->m_gb_camera->Anchor = static_cast<System::Windows::Forms::AnchorStyles>(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Left)
				| System::Windows::Forms::AnchorStyles::Right));
			this->m_gb_camera->Controls->Add(this->m_lb_pt3);
			this->m_gb_camera->Controls->Add(this->m_lb_pt2);
			this->m_gb_camera->Controls->Add(this->m_lb_pt1);
			this->m_gb_camera->Controls->Add(this->m_tb_ip_p2);
			this->m_gb_camera->Controls->Add(this->m_tb_ip_p4);
			this->m_gb_camera->Controls->Add(this->m_tb_ip_p3);
			this->m_gb_camera->Controls->Add(this->m_tb_ip_p1);
			this->m_gb_camera->Controls->Add(this->m_lb_ip);
			this->m_gb_camera->Controls->Add(this->m_btn_disconnect);
			this->m_gb_camera->Controls->Add(this->m_btn_connect);
			this->m_gb_camera->Location = System::Drawing::Point(3, 12);
			this->m_gb_camera->Name = L"m_gb_camera";
			this->m_gb_camera->Size = System::Drawing::Size(173, 122);
			this->m_gb_camera->TabIndex = 2;
			this->m_gb_camera->TabStop = false;
			this->m_gb_camera->Text = L"Camera";
			// 
			// m_lb_pt3
			// 
			this->m_lb_pt3->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 8.25F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point,
				static_cast<System::Byte>(0)));
			this->m_lb_pt3->Location = System::Drawing::Point(128, 43);
			this->m_lb_pt3->Name = L"m_lb_pt3";
			this->m_lb_pt3->Size = System::Drawing::Size(10, 15);
			this->m_lb_pt3->TabIndex = 27;
			this->m_lb_pt3->Text = L".";
			this->m_lb_pt3->TextAlign = System::Drawing::ContentAlignment::TopRight;
			// 
			// m_lb_pt2
			// 
			this->m_lb_pt2->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 8.25F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point,
				static_cast<System::Byte>(0)));
			this->m_lb_pt2->Location = System::Drawing::Point(86, 43);
			this->m_lb_pt2->Name = L"m_lb_pt2";
			this->m_lb_pt2->Size = System::Drawing::Size(10, 15);
			this->m_lb_pt2->TabIndex = 26;
			this->m_lb_pt2->Text = L".";
			this->m_lb_pt2->TextAlign = System::Drawing::ContentAlignment::TopRight;
			// 
			// m_lb_pt1
			// 
			this->m_lb_pt1->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 8.25F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point,
				static_cast<System::Byte>(0)));
			this->m_lb_pt1->Location = System::Drawing::Point(44, 43);
			this->m_lb_pt1->Name = L"m_lb_pt1";
			this->m_lb_pt1->Size = System::Drawing::Size(10, 15);
			this->m_lb_pt1->TabIndex = 25;
			this->m_lb_pt1->Text = L".";
			this->m_lb_pt1->TextAlign = System::Drawing::ContentAlignment::TopRight;
			// 
			// m_tb_ip_p2
			// 
			this->m_tb_ip_p2->Location = System::Drawing::Point(55, 38);
			this->m_tb_ip_p2->Name = L"m_tb_ip_p2";
			this->m_tb_ip_p2->Size = System::Drawing::Size(29, 20);
			this->m_tb_ip_p2->TabIndex = 2;
			this->m_tb_ip_p2->Text = L"168";
			this->m_tb_ip_p2->TextAlign = System::Windows::Forms::HorizontalAlignment::Center;
			this->m_tb_ip_p2->KeyPress += gcnew System::Windows::Forms::KeyPressEventHandler(this, &ui_main::m_tb_ip_p2_KeyPress);
			this->m_tb_ip_p2->Leave += gcnew System::EventHandler(this, &ui_main::m_tb_ip_p2_Leave);
			// 
			// m_tb_ip_p4
			// 
			this->m_tb_ip_p4->Location = System::Drawing::Point(138, 38);
			this->m_tb_ip_p4->Name = L"m_tb_ip_p4";
			this->m_tb_ip_p4->Size = System::Drawing::Size(29, 20);
			this->m_tb_ip_p4->TabIndex = 4;
			this->m_tb_ip_p4->Text = L"254";
			this->m_tb_ip_p4->TextAlign = System::Windows::Forms::HorizontalAlignment::Center;
			this->m_tb_ip_p4->KeyPress += gcnew System::Windows::Forms::KeyPressEventHandler(this, &ui_main::m_tb_ip_p4_KeyPress);
			this->m_tb_ip_p4->Leave += gcnew System::EventHandler(this, &ui_main::m_tb_ip_p4_Leave);
			// 
			// m_tb_ip_p3
			// 
			this->m_tb_ip_p3->Location = System::Drawing::Point(97, 38);
			this->m_tb_ip_p3->Name = L"m_tb_ip_p3";
			this->m_tb_ip_p3->Size = System::Drawing::Size(29, 20);
			this->m_tb_ip_p3->TabIndex = 3;
			this->m_tb_ip_p3->Text = L"100";
			this->m_tb_ip_p3->TextAlign = System::Windows::Forms::HorizontalAlignment::Center;
			this->m_tb_ip_p3->KeyPress += gcnew System::Windows::Forms::KeyPressEventHandler(this, &ui_main::m_tb_ip_p3_KeyPress);
			this->m_tb_ip_p3->Leave += gcnew System::EventHandler(this, &ui_main::m_tb_ip_p3_Leave);
			// 
			// m_tb_ip_p1
			// 
			this->m_tb_ip_p1->Location = System::Drawing::Point(12, 38);
			this->m_tb_ip_p1->Name = L"m_tb_ip_p1";
			this->m_tb_ip_p1->Size = System::Drawing::Size(29, 20);
			this->m_tb_ip_p1->TabIndex = 1;
			this->m_tb_ip_p1->Text = L"192";
			this->m_tb_ip_p1->TextAlign = System::Windows::Forms::HorizontalAlignment::Center;
			this->m_tb_ip_p1->KeyPress += gcnew System::Windows::Forms::KeyPressEventHandler(this, &ui_main::m_tb_ip_p1_KeyPress);
			this->m_tb_ip_p1->Leave += gcnew System::EventHandler(this, &ui_main::m_tb_ip_p1_Leave);
			// 
			// m_lb_ip
			// 
			this->m_lb_ip->AutoSize = true;
			this->m_lb_ip->Location = System::Drawing::Point(9, 22);
			this->m_lb_ip->Name = L"m_lb_ip";
			this->m_lb_ip->Size = System::Drawing::Size(61, 13);
			this->m_lb_ip->TabIndex = 3;
			this->m_lb_ip->Text = L"IP Address:";
			// 
			// m_btn_disconnect
			// 
			this->m_btn_disconnect->Anchor = static_cast<System::Windows::Forms::AnchorStyles>((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Right));
			this->m_btn_disconnect->Location = System::Drawing::Point(87, 93);
			this->m_btn_disconnect->Name = L"m_btn_disconnect";
			this->m_btn_disconnect->Size = System::Drawing::Size(75, 23);
			this->m_btn_disconnect->TabIndex = 6;
			this->m_btn_disconnect->Text = L"Disconnect";
			this->m_btn_disconnect->UseVisualStyleBackColor = true;
			this->m_btn_disconnect->Click += gcnew System::EventHandler(this, &ui_main::m_btn_disconnect_Click);
			// 
			// m_btn_connect
			// 
			this->m_btn_connect->Anchor = static_cast<System::Windows::Forms::AnchorStyles>((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Right));
			this->m_btn_connect->Location = System::Drawing::Point(87, 64);
			this->m_btn_connect->Name = L"m_btn_connect";
			this->m_btn_connect->Size = System::Drawing::Size(75, 23);
			this->m_btn_connect->TabIndex = 5;
			this->m_btn_connect->Text = L"Connect";
			this->m_btn_connect->UseVisualStyleBackColor = true;
			this->m_btn_connect->Click += gcnew System::EventHandler(this, &ui_main::m_btn_connect_Click);
			// 
			// m_panel_image
			// 
			this->m_panel_image->Controls->Add(this->m_pb_image);
			this->m_panel_image->Dock = System::Windows::Forms::DockStyle::Fill;
			this->m_panel_image->Location = System::Drawing::Point(180, 0);
			this->m_panel_image->Name = L"m_panel_image";
			this->m_panel_image->Size = System::Drawing::Size(795, 754);
			this->m_panel_image->TabIndex = 1;
			// 
			// m_pb_image
			// 
			this->m_pb_image->Anchor = static_cast<System::Windows::Forms::AnchorStyles>((((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Bottom)
				| System::Windows::Forms::AnchorStyles::Left)
				| System::Windows::Forms::AnchorStyles::Right));
			this->m_pb_image->BackColor = System::Drawing::Color::DarkGray;
			this->m_pb_image->Location = System::Drawing::Point(0, 3);
			this->m_pb_image->Name = L"m_pb_image";
			this->m_pb_image->Size = System::Drawing::Size(792, 748);
			this->m_pb_image->TabIndex = 0;
			this->m_pb_image->TabStop = false;
			// 
			// m_thread_read_image
			// 
			this->m_thread_read_image->DoWork += gcnew System::ComponentModel::DoWorkEventHandler(this, &ui_main::m_thread_read_image_DoWork);
			this->m_thread_read_image->RunWorkerCompleted += gcnew System::ComponentModel::RunWorkerCompletedEventHandler(this, &ui_main::m_thread_read_image_RunWorkerCompleted);
			// 
			// ui_main
			// 
			this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
			this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
			this->ClientSize = System::Drawing::Size(975, 754);
			this->Controls->Add(this->m_panel_image);
			this->Controls->Add(this->m_panel_controls);
			this->Name = L"ui_main";
			this->Text = L"CamRecord-Sprinter Demo";
			this->m_panel_controls->ResumeLayout(false);
			this->m_gb_display->ResumeLayout(false);
			this->m_gb_display->PerformLayout();
			this->m_gb_properties->ResumeLayout(false);
			this->m_gb_properties->PerformLayout();
			this->m_gb_control->ResumeLayout(false);
			this->m_gb_camera->ResumeLayout(false);
			this->m_gb_camera->PerformLayout();
			this->m_panel_image->ResumeLayout(false);
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^>(this->m_pb_image))->EndInit();
			this->ResumeLayout(false);

		}
#pragma endregion
	
	private: System::Void m_btn_connect_Click(System::Object^  sender, System::EventArgs^  e) {

				 if (m_camera != nullptr)
				 {
					 System::String^  ip_address;

					 // Build ip address
					 ip_address = m_tb_ip_p1->Text + "." + m_tb_ip_p2->Text + "." + m_tb_ip_p3->Text + "." + m_tb_ip_p4->Text;

					 // Connect to camera
					 m_connected = m_camera->Connect(ip_address);		

					 // If connected then read sensor parameter
					 if (m_connected)
					 {
						 m_camera->GetSensorScale(m_x_min, m_x_max, m_x_inc, m_y_min, m_y_max, m_y_inc);
					 }
				 }

				 ActualizeFormStatus();
				 ActualizeControls();
	}

	private: System::Void m_btn_disconnect_Click(System::Object^  sender, System::EventArgs^  e) {

				 if (m_camera != nullptr)
				 {
					 if (m_connected)
					 {
						 m_camera->Disconnect();
					 }

					 m_connected = false;
				 }

				 ActualizeFormStatus();
				 ActualizeControls();
	}

	private: System::Void m_btn_live_start_Click(System::Object^  sender, System::EventArgs^  e) {

				 if (m_camera != nullptr)
				 {
					 System::Drawing::Bitmap ^displayed_image(nullptr), ^new_image(nullptr);
					 
					 // Get current image size
					 m_camera->GetResolution(m_image_width, m_image_height);

					 // Save pointer to the current displayed bitmap
					 if (m_pb_image->Image)
					 {
						 displayed_image = (System::Drawing::Bitmap^)m_pb_image->Image;
					 }					 

					 // Create new bitmap object
					 new_image = gcnew System::Drawing::Bitmap(m_image_width, m_image_height, Imaging::PixelFormat::Format8bppIndexed);

					 // Create grey scale color palette
					 Imaging::ColorPalette^ palette = new_image->Palette;
					 for (int i = 0; i < 256; ++i)
					 {
						 palette->Entries[i] = System::Drawing::Color::FromArgb(i, i, i);
					 }

					 new_image->Palette = palette;

					 // Set the new image to the picture box and delete the old
					 m_image = new_image;
					 m_pb_image->Image = m_image;

					 if (displayed_image)
					 {
						 delete displayed_image;
					 }

					 // Start camera live mode
					 m_live_mode = m_camera->LiveStart(false);

					 // Start thread to read and display the images
					 if (m_live_mode)
					 {
						 m_stop_thread = false;
						 m_thread_read_image->RunWorkerAsync();
					 }
				 }
				 
				 ActualizeFormStatus();
	}

	private: System::Void m_btn_live_stop_Click(System::Object^  sender, System::EventArgs^  e) {

				 if (m_camera != nullptr)
				 {
					 // Stop thread to read and display the images
					 m_stop_thread = true;

					 // Wait until the live thread is stopped (max. time to wait is 2s)
					 System::Int32 t1 = 0;
					 System::Int32 t2 = 0;

					 t1 = System::Environment::TickCount;

					 do
					 {
						 Application::DoEvents();
						 System::Threading::Thread::Sleep(10);
						 t2 = System::Environment::TickCount;
					 } while (((t2 - t1) < 2000) && !m_thread_stopped);

					 // Stop camera live mode
					 m_camera->LiveStop();

					 m_live_mode = false;
				 }

				 ActualizeFormStatus();
	}

	private: System::Void m_tb_ip_p1_KeyPress(System::Object^  sender, System::Windows::Forms::KeyPressEventArgs^  e) {

				 // Validate text box input
				 e->Handled = !System::Char::IsDigit(e->KeyChar) && !System::Char::IsControl(e->KeyChar);
	}

	private: System::Void m_tb_ip_p2_KeyPress(System::Object^  sender, System::Windows::Forms::KeyPressEventArgs^  e) {

				 // Validate text box input
				 e->Handled = !System::Char::IsDigit(e->KeyChar) && !System::Char::IsControl(e->KeyChar);
	}

	private: System::Void m_tb_ip_p3_KeyPress(System::Object^  sender, System::Windows::Forms::KeyPressEventArgs^  e) {

				 // Validate text box input
				 e->Handled = !System::Char::IsDigit(e->KeyChar) && !System::Char::IsControl(e->KeyChar);
	}

	private: System::Void m_tb_ip_p4_KeyPress(System::Object^  sender, System::Windows::Forms::KeyPressEventArgs^  e) {
				 
				 // Validate text box input
				 e->Handled = !System::Char::IsDigit(e->KeyChar) && !System::Char::IsControl(e->KeyChar);
	}

	private: System::Void m_tb_ip_p1_Leave(System::Object^  sender, System::EventArgs^  e) {

				 // Validate text box input
				 System::Int32 value(0);

				 if (m_tb_ip_p1->Text->Length > 0)
				 {
					 value = System::Convert::ToInt32(m_tb_ip_p1->Text);

					 if (value > 255)
					 {
						 m_tb_ip_p1->Text = "255";
					 }
				 }
				 else
				 {
					 m_tb_ip_p1->Text = "0";
				 }
	}

	private: System::Void m_tb_ip_p2_Leave(System::Object^  sender, System::EventArgs^  e) {

				 // Validate text box input
				 System::Int32 value(0);

				 if (m_tb_ip_p2->Text->Length > 0)
				 {
					 value = System::Convert::ToInt32(m_tb_ip_p2->Text);

					 if (value > 255)
					 {
						 m_tb_ip_p2->Text = "255";
					 }
				 }
				 else
				 {
					 m_tb_ip_p2->Text = "0";
				 }
	}

	private: System::Void m_tb_ip_p3_Leave(System::Object^  sender, System::EventArgs^  e) {

				 // Validate text box input
				 System::Int32 value(0);

				 if (m_tb_ip_p3->Text->Length > 0)
				 {
					 value = System::Convert::ToInt32(m_tb_ip_p3->Text);

					 if (value > 255)
					 {
						 m_tb_ip_p3->Text = "255";
					 }
				 }
				 else
				 {
					 m_tb_ip_p3->Text = "0";
				 }
	}

	private: System::Void m_tb_ip_p4_Leave(System::Object^  sender, System::EventArgs^  e) {

				 // Validate text box input
				 System::Int32 value(0);

				 if (m_tb_ip_p4->Text->Length > 0)
				 {
					 value = System::Convert::ToInt32(m_tb_ip_p4->Text);

					 if (value > 255)
					 {
						 m_tb_ip_p4->Text = "255";
					 }
				 }
				 else
				 {
					 m_tb_ip_p4->Text = "0";
				 }
	}

	private: System::Void m_tb_x_KeyPress(System::Object^  sender, System::Windows::Forms::KeyPressEventArgs^  e) {

				 // Validate text box input
				 e->Handled = !System::Char::IsDigit(e->KeyChar) && !System::Char::IsControl(e->KeyChar);
	}
			 
	private: System::Void m_tb_y_KeyPress(System::Object^  sender, System::Windows::Forms::KeyPressEventArgs^  e) {

				 // Validate text box input
				 e->Handled = !System::Char::IsDigit(e->KeyChar) && !System::Char::IsControl(e->KeyChar);
	}

	private: System::Void m_tb_exposure_KeyPress(System::Object^  sender, System::Windows::Forms::KeyPressEventArgs^  e) {

				 // Validate text box input
				 e->Handled = !System::Char::IsDigit(e->KeyChar) && !System::Char::IsControl(e->KeyChar);
	}

	private: System::Void m_tb_fps_KeyPress(System::Object^  sender, System::Windows::Forms::KeyPressEventArgs^  e) {

				 // Validate text box input
				 e->Handled = !System::Char::IsDigit(e->KeyChar) && !System::Char::IsControl(e->KeyChar);
	}

	private: System::Void SetResolution_Click(System::Object^  sender, System::EventArgs^  e) {

				 if (m_camera != nullptr)
				 {
					 if ((m_tb_x->Text->Length > 0) && (m_tb_y->Text->Length > 0))
					 {
						 System::Int32 res_x(0), res_y(0);

						 res_x = System::Convert::ToInt32(m_tb_x->Text);
						 res_y = System::Convert::ToInt32(m_tb_y->Text);

						 // Correct resolution if new value is not in a valid increment
						 res_x -= (res_x % m_x_inc);
						 res_y -= (res_y % m_y_inc);
						 
						 // Limit to the min. and max. value
						 res_x = System::Math::Min(res_x, m_x_max);
						 res_x = System::Math::Max(res_x, m_x_min);

						 res_y = System::Math::Min(res_y, m_y_max);
						 res_y = System::Math::Max(res_y, m_y_min);
						 
						 // Change camera resolution
						 m_camera->SetResolution(res_x, res_y);

						 ActualizeControls();
					 }					 
				 }
	}

	private: System::Void m_btn_exposure_send_Click(System::Object^  sender, System::EventArgs^  e) {

				 if (m_camera != nullptr)
				 {
					 if (m_tb_exposure->Text->Length > 0)
					 {
						 System::Int32 exposure(0);

						 exposure = System::Convert::ToInt32(m_tb_exposure->Text) * 1000;

						 m_camera->SetExposureTime(exposure);

						 ActualizeControls();
					 }
				 }
	}

	private: System::Void m_btn_fps_send_Click(System::Object^  sender, System::EventArgs^  e) {

				 if (m_camera != nullptr)
				 {
					 if (m_tb_fps->Text->Length > 0)
					 {
						 System::Int32 framerate(0);

						 framerate = System::Convert::ToInt32(m_tb_fps->Text);

						 m_camera->SetFramerate(framerate);

						 ActualizeControls();
					 }
				 }
	}

	private: System::Void m_thread_read_image_DoWork(System::Object^  sender, System::ComponentModel::DoWorkEventArgs^  e) {

				 m_stop_thread		= false;
				 m_thread_stopped	= false;

				 while (!m_stop_thread)
				 {
					 // Read and display the image
					 ReadImage();

					 // Handle windows events and sleep for 50 ms
					 Application::DoEvents();
					 System::Threading::Thread::Sleep(50);
				 }
	}

	private: System::Void m_thread_read_image_RunWorkerCompleted(System::Object^  sender, System::ComponentModel::RunWorkerCompletedEventArgs^  e) {

				 // Set flag that tread is stopped
				 m_thread_stopped = true;
	}

	private: System::Void m_cb_viewmode_SelectedIndexChanged(System::Object^  sender, System::EventArgs^  e) {

				 // Change view mode of the picture box
				 switch (m_cb_viewmode->SelectedIndex)
				 {
					case 0:
					{
						m_pb_image->SizeMode = System::Windows::Forms::PictureBoxSizeMode::Normal;
						break;
					}

					case 1:
					{
						m_pb_image->SizeMode = System::Windows::Forms::PictureBoxSizeMode::CenterImage;
						break;
					}

					case 2:
					{
						m_pb_image->SizeMode = System::Windows::Forms::PictureBoxSizeMode::StretchImage;
						break;
					}
				 }
	}
};
}
